# Generated by Django 2.2.12 on 2020-07-11 12:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('expense', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='expense',
            name='notes',
            field=models.TextField(blank=True, null=True, verbose_name='Notes'),
        ),
        migrations.AlterField(
            model_name='expense',
            name='slug',
            field=models.SlugField(blank=True, help_text='For fast recall', unique=True, verbose_name='Identifier slug'),
        ),
        migrations.AlterField(
            model_name='expense',
            name='name',
            field=models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Name'),
        ),
    ]
